function d = dp2dbase(m,dpack,varargin)
%
% DP2DBASE  Datapack to database converstion (cell array to struct).
%
% Syntax:
%   d = dp2dbase(m,dpack,...)
% Arguments:
%   d struct, m model, dpack cell
% Options:
%   'merge' struct (empty)
%   'include' logical (false)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'dbase',[],...
  'merge',struct(),...
  'include',false,...
};
options = passopt(default,varargin{:});

if isstruct(options.dbase), options.merge = options.dbase; end

% function body ---------------------------------------------------------------------------------------------

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);

% number of databases and number of parameterizations
if size(dpack,3) ~= 1 && size(dpack,3) ~= size(m.assign,3), error_(44); end

id = m.solutionid{1};
dpack{1}(m.log(real(id)),:,:) = exp(dpack{1}(m.log(real(id)),:,:));
mat = dpack{1};
name = m.name(real(id));
shift = imag(id);
cmt = m.namelabel(real(id));

id = m.solutionid{2};
if size(dpack{2},1) < nx, id(imag(id) ~= 0) = []; end % reduced datapack
dpack{2}(m.log(real(id)),:,:) = exp(dpack{2}(m.log(real(id)),:,:));
mat = [mat;dpack{2}];
name = [name,m.name(real(id))];
shift = [shift,imag(id)];
cmt = [cmt,m.namelabel(real(id))];

id = m.solutionid{3};
mat = [mat;dpack{3}];
name = [name,m.name(real(id))];
shift = [shift,imag(id)];
cmt = [cmt,m.namelabel(real(id))];

d = mat2db(options.merge,mat,dpack{4},name,shift,cmt,options.include);

end % of primary function -----------------------------------------------------------------------------------