function [Theta,Upsilon,list] = fevd(m,nper,varargin)
%
% FEVD  Forecast error variance decomposition.
%
% Syntax:
%   [Theta,Thetarel,list] = fevd(m,nper,...)
% Arguments:
%   Theta numeric, Upsilon numeric, list cellstr, m model, nper numeric
% Options:
%   'select' cellstr|char|Inf
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.select), options.select = {options.select}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
Theta = zeros([ny+nx,ne,nper,nalt]);
Upsilon = zeros([ny+nx,ne,nper,nalt]);

[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
  [Theta(:,:,:,ialt),Upsilon(:,:,:,ialt)] = fevd(T,R,K,Z,H,D,Za,Omega,nper);
end

% print variable names
if nargout > 2 || iscellstr(options.select)
  ny = size(Z,1);
  nx = size(T,1);
  tmp = printid_(m);
  list = {tmp(1:ny+nx),tmp(ny+nx+1:end)};
end

% select variables
if iscellstr(options.select)
  [Theta,index,list,notfound] = select(Theta,list,options.select);
  if nargout > 1, Upsilon = Upsilon(index{1},index{2},:,:); end
  if ~isempty(notfound), warning_(1,notfound); end
end

end % of primary function -----------------------------------------------------------------------------------