function [F,list] = ffrf(m,freq,varargin)
%
% FFRF  Frequency response function of state-space filter.
%
% Syntax:
%   [F,list] = ffrf(s,freq,...)
% Arguments:
%   F cell, list cellstr, m model, freq numeric
% Options:
%   'maxiter' numeric (500)
%   'select' cellstr|char|Inf (Inf)
%   'tolerance' numeric (1e-7)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'maxiter',500,...
  'select',Inf,...
  'tolerance',1e-7,...
};
options = passopt(default,varargin{:});

if ischar(options.select), options.select = {options.select}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

freq = vech(freq);
nfreq = length(freq);
[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
F = {zeros([nx,ny,nfreq,nalt]),freq};

if ny > 0
  [flag,index] = isnan(m,'solution');
  for ialt = find(~index)
    [T,R,K,Z,H,D,Za,Omega] = sspace_(m,ialt);
    F{1}(:,:,:,ialt) = ffrf(T,R,K,Z,H,D,Za,Omega,freq,options.tolerance,options.maxiter);
  end
end

% print variable names
if nargout > 1 || iscellstr(options.select)
  tmp = printid_(m);
  list = {tmp(ny+(1:nx)),tmp(1:ny)};
end

% select variables
if iscellstr(options.select)
  [F{1},index,list,notfound] = select(F{1},list,options.select);
  if ~isempty(notfound), warning_(1,notfound); end
else

end % of primary function -----------------------------------------------------------------------------------