function m = model(varargin)
%
% MODEL  Read in model code file and create new model object.
%
% Syntax:
%   m = model(fname,...)
% Arguments:
%   m model; fname char
% Options:
%   'assign' struct (empty) Assign parameter values from database.
%   'linear' logical (false) Declare that the model is linear.
%   'nalt' numeric (1) Set the number of altenative parameterizations.
%   'precision' char ('double') Store system matrices at double or single precision.
%   'std' numeric (1) Default std error of model residuals.
%   'userdata' ? (empty) User data associated with model object.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'assign',struct(),...
  'linear',false,...
  'nalt',1,...
  'precision','double',...
  'std',1,...
  'userdata',[],...
};
options = passopt(default,varargin{2:end});

% function body ---------------------------------------------------------------------------------------------

if nargin == 0
  m = class(empty_(),'model'); 
elseif nargin == 1 && ismodel(varargin{1})
  m = varargin{1};
elseif nargin == 1 && isstruct(varargin{1})
  m = class(empty_(),'model'); 
  for f = vech(fieldnames(m))
    try, m.(f{1}) = varargin{1}.(f{1}); catch, end
  end
elseif nargin > 0 && ischar(varargin{1})
  m = class(empty_(),'model');
  m.fname = varargin{1};
  m.linear = options.linear;
  [m,options.assign] = parser_(m,options.assign);
  if m.linear == false, m = reorder_(m); end
  m = meta_(m);
  try, defaultstd = assign.std_(1);
    catch, defaultstd = options.std; end
  m = reset(m,defaultstd);
  if ~isempty(options.assign), m = assign(m,options.assign); end
  if ~isempty(options.userdata), m = set(m,'userdata',options.userdata); end
  if round(options.nalt) ~= 1, m = nalt_(m,options.nalt); end
else
  error('Incorrect number or type of input argument(s).')
end

% end of function body --------------------------------------------------------------------------------------

  function m = empty_() % nested function -------------------------------------------------------------------
  
  m.IRIS_MODEL = true;
  m.fname = '';
  m.linear = false;
  if strcmpi(strtrim(options.precision),'single'), m.precision = 'single';
    else, m.precision = 'double'; end
  m.assign = nan(0,m.precision);
  m.assign0 = nan(0,m.precision);

  [m.name,m.namelabel] = deal({});
  m.log = false([1,0]);
  [m.eqtn,m.eqtnS,m.eqtnF,m.eqtnlabel] = deal({});
  m.outside = struct('label',{{}},'lhs',{{}},'rhs',{{}});
  m.mapoftunes = struct([]);
  [m.nametype,m.eqtntype] = deal(zeros([1,0]));
  m.nonlin = false([1,0]);
  m.occur = false([0,0,0]);
  m.tzero = NaN;
  
  m.ixname = cell([1,6]);
  m.nname =  nan([1,6]);
  m.ixeqtn = cell([1,2]);
  m.neqtn = nan([1,2]);
  [m.nameorder,m.eqtnorder] = deal(zeros([1,0]));
  
  m.systemid = {};
  [m.metaderiv,m.metasystem,m.systemident] = deal(struct);
  m.metadelete = [];  
  m.deriv0 = [];
  m.system0 = [];
  m.eigval = [];
  m.diffuse = {};

  m.expand = {};
  m.solution(1:7) = {[]};
  m.solutionid = {{},{},{}};
  m.icondix = [];

  m.userdata = [];

  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------
 