function text = readcode_(fname,varargin)
%
% @MODEL/PRIVATE/READCODE_  Read in and adjust model code file.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin == 1 || isempty(varargin{1}), assign = struct;
  else assign = varargin{1}; end

text = file2char(fname);

% stop at @stop %
tmp = strfind(text,'@stop');
if ~isempty(tmp), text = text(1:tmp(1)-1); end  

% remove CR characters %
text = strrep(text,char(13),'');

% remove line comments %
text = regexprep(text,'%.*?\n','\n');
text = regexprep(text,'%.*?$','');

% remove block comments %
text = strrep(text,'/*',char(1));
text = strrep(text,'*/',char(2));
textlength = NaN;
while length(text) ~= textlength
  textlength = length(text);  
  text = regexprep(text,'\x{1}[^\x{1}]*?\x{2}','');
end
text = strrep(text,char(1),'/*');
text = strrep(text,char(2),'*/');

% evaluate @if..@else..@end %
text = convertif_(text,true);
patt = '\x{1}\s*([^\n;]*?)[\n;]([^\x{1}]*?)(\x{2})?(?(3)[^\x{1}]*?|)\x{3}';
invalid = cell([1,0]);
[tokens,start,finish] = regexp(text,patt,'tokens','start','end','once');
while ~isempty(tokens)
  [value,valid] = eval_(assign,tokens{1});
  repl = iff(value,tokens{2},tokens{4});  
  if valid == false, invalid{end+1} = convertif_(tokens{1},false); end
  text = [text(1:start-1),repl,text(finish+1:end)];
  [tokens,start,finish] = regexp(text,patt,'tokens','start','end','once');
end
if ~isempty(invalid), warning_(30,invalid); end
text = convertif_(text,false);

% characters beyond char(255) %
index = text > char(255);
if any(index), error_(40,{text(index)}); end

% create codes for @ar processes %
patt = '@ar\((.*?)\)';
tokens = regexp(text,patt,'tokens','once');
while ~isempty(tokens)
  repl = ar_(tokens{1});
  text = regexprep(text,patt,repl,'once');
  tokens = regexp(text,patt,'tokens','once');
end

% include/input files
patt = '@include\((.*?)\)|@input\((.*?)\)';
tokens = regexp(text,patt,'tokens','once');
while ~isempty(tokens)
  repl = readcode_(tokens{1},assign);
  text = regexprep(text,patt,repl,'once');
  tokens = regexp(text,patt,'tokens','once');
end

  function text = convertif_(text,direction) % nested function ----------------------------------------------
    if direction == true
      text = strrep(text,'@if',char(1));
      text = strrep(text,'@else',char(2));
      text = strrep(text,'@end',char(3));
    else
      text = strrep(text,char(1),'@if');
      text = strrep(text,char(2),'@else');
      text = strrep(text,char(3),'@end');
    end
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------
  
  function [value,valid] = eval_(assign,expr) % subfunction -------------------------------------------------
  expr = strrep(expr,'@','');
  expr = regexprep(expr,'\<inf\>','inf()','ignorecase');
  expr = regexprep(expr,'\<nan\>','nan()','ignorecase');
  expr = regexprep(expr,'\<true\>','true()','ignorecase');
  expr = regexprep(expr,'\<false\>','false()','ignorecase');
  expr = regexprep(expr,'([A-Za-z]\w*)(?![\w\(])','assign.$1');
  try, value = eval(expr);
    catch, value = NaN; end
  if ~islogical(value) || length(value) ~= 1
    valid = false;
    value = false;
  else
    valid = true;
  end
  end % of subfunction --------------------------------------------------------------------------------------

  function repl = ar_(argument,assign) % subfunction --------------------------------------------------------
  % arguments: name,mean,autocorrelation,residual,transform %
  list = regexp(argument,'[^,]+','match');
  list = strtrim(list);
  nargument = length(list);
  if nargument == 5 && isempty(list{5})
    list = list(1:4);
    nargument = 4;
  end
  if all(nargument ~= 4:5) , error_(41,{argument}); end
  if nargument == 4
    list{1} = sprintf('%s{%%s}',list{1});
  else
    list{1} = sprintf('%s(%s{%%s})',list{5},list{1});
    list{2} = sprintf('%s(%s)',list{5},list{2});
  end
  repl = sprintf('%s = %s*%s + (1-%s)*%s + %s',list{1},list{3},list{1},list{3},list{2},list{4});
  repl = sprintf(repl,'0','-1');
  end % of subfunction --------------------------------------------------------------------------------------