function [A,B,K,Omega] = companion(w)
%
% COMPANION  System matrices and data for 1st-order VAR companion form.
%
% Syntax:
%   [A,B,K,Omega] = companion(w)
% Arguments:
%   A numeric; B numeric; K numeric; Omega numeric; w rvar|svar
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);

if p == 0
  A = zeros([ny,ny,nalt]);
else
  for ialt = 1 : nalt, A(:,:,ialt) = [w.A(:,:,ialt);eye([ny*(p-1),ny*p])]; end 
end

if nargout > 1
  B = get(w,'B');
  B = [B;zeros([ny*(p-1),ny,nalt])];
end
if nargout > 2
  K = [w.K(:,1,:);zeros([ny*(p-1),1,nalt])];
end
if nargout > 3
  Omega = get(w,'Omega');
end

end % -------------------------------------------------------------------------------------------------------