function [Theta,Upsilon] = fevd(w,nper)
%
% FEVD  Forecast error variance decomposition.
%
% Syntax:
%   [Theta,Upsilon] = fevd(w,nper)
% Arguments:
%   Theta numeric; Upsilon numeric; w svar; nper numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if isempty(w.B), error_(13); end

[ny,p,nalt] = dim(w);
[T,R,K,Z,H,D,U,Omega] = sspace(w);
for ialt = 1 : nalt
  [theta,upsilon] = fevd(T(:,:,ialt),R(:,:,ialt),[],[],[],[],U(:,:,ialt),Omega(:,:,ialt),nper);
  Theta(:,:,:,ialt) = theta(1:ny,:,:);
  if nargout > 1, Upsilon(:,:,:,ialt) = upsilon(1:ny,:,:); end
end

end % of primary function -----------------------------------------------------------------------------------