function [w,dpack] = integrate(w,dpack,varargin)
%
% INTEGRATE  Integrate VAR process.
%
% Syntax:
%   w = integrate(w)
%   [w,data] = integrate(w,data,...)
% Arguments:
%   w rvar|svar
% Options:
%   'applyto' logical|numeric (true([1,ny]))
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

[ny,p,nalt] = dim(w);

default = {
  'applyto',true([1,ny]),...
  'select',[],...
};
options = passopt(default,varargin{:});

if ~isempty(options.select), options.applyto = options.select; end

% function body ---------------------------------------------------------------------------------------------

if ~islogical(options.applyto) && ~isinf1(options.applyto)
  aux = options.applyto;
  options.applyto = false([1,ny]);
  options.applyto(aux) = true;
end

if any(options.applyto)
  D = cat(3,eye(ny),-eye(ny));
  D(~options.applyto,~options.applyto,2) = 0;
  A = w.A;
  w.A(:,end+1:end+ny,:) = NaN;
  for ialt = 1 : nalt
    a = polyprod(var2poly(A(:,:,ialt)),D);
    w.A(:,:,ialt) = poly2var(a);
  end
  w.TZ = {};
  w.eigval = [];
end

% integrate datapack series
if nargin > 1 && nargout > 1 && ~isempty(dpack) 
  ndata = size(dpack{1},3);
  dpack{1} = [nan([ny,1,nalt]),dpack{1}];
  dpack{1}(options.applyto,1,:) = 0;
  dpack{1}(options.applyto,:,:) = cumsum(dpack{1}(options.applyto,:,:),2);
  dpack{3} = [nan([ny,1,nalt]),dpack{3}];
  dpack{4} = [dpack{4}(1)-1,dpack{4}];
  dpack{2} = zeros([0,length(dpack{4})]);
else
  dpack = [];
end

end % of primary function -----------------------------------------------------------------------------------