function [A,K,Omega,nhyper,E,RR,rr,count] = glsq_(w,y,constant,p,polyorder,restrict,cointeg,tolerance,maxiter)
%
% RVAR/PRIVATE/GLSQ_  Generalized least squares for reduced-form VAR
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,void,nalt] = size(y);
% include constant into cointegrating vector
if ~isempty(cointeg) && size(cointeg,1) < ny+1, cointeg(size(cointeg,1)+1:ny+1,:) = 0; end
if ~isempty(cointeg), p = p - 1; end

[y0,y1,k0,g1] = stackdata(y,p,constant,cointeg);
nper = size(y0,2);
nk = size(k0,1);
ng = size(g1,1);

% polynomial distribution (applied to y1 parameters only)
if 1 + polyorder < p || ~isinf(polyorder), [RR0,CC0] = pdlrestrict_(ny,nk,ng,p,polyorder);
  else RR0 = []; CC0 = []; end

% parameter restrictions
[RR,rr,nhyper] = restrict_(restrict,RR0,CC0,ny,nk,ng,p);

for ialt = 1 : nalt
  % ordinary least squares
  M = [y1(:,:,ialt);k0;g1(:,:,ialt)];
  H = y0(:,:,ialt) / M;
  Ei = y0(:,:,ialt) - H*M;
  Omegai = Ei*transpose(Ei) / nper;
  count(ialt) = 1;
  % generalized least squares
  if ~isempty(RR)
    maxdiff = Inf;
    while maxdiff > tolerance && count(ialt) <= maxiter
      H0 = H;
      Omega_ = inv(Omegai);
      z = vec(y0(:,:,ialt)) - kron(transpose(M),eye(ny))*rr;
      hyper = inv(transpose(RR)*kron(M*transpose(M),Omega_)*RR)*transpose(RR)*kron(M,Omega_)*z;
      H = reshape(RR*hyper + rr,[ny,ny*p+nk+ng]);
      Ei = y0(:,:,ialt) - H*M;
      Omegai = Ei*transpose(Ei) / nper;
      maxdiff = max(vec(abs(H - H0)));
      count(ialt) = count(ialt) + 1;
    end
  end
  Ai = H(:,1:end-nk-ng);
  H(:,1:end-nk-ng) = [];
  % constant
  if nk > 0, Ki = H(:,1); H(:,1) = []; else, Ki = zeros([ny,1]); end
  % VEC to VAR conversion
  if ng > 0
    G = H(:,1:ng);
    aux = polyprod(reshape(Ai,[ny,ny,p]),cat(3,eye(ny),-eye(ny)));
    aux = polysum(aux,eye(ny)+G*transpose(cointeg(1:ny,:)));
    Ai = reshape(aux,[ny,ny*(p+1)]);
    Ki = Ki + G*transpose(cointeg(end,:));
  end
A(:,:,ialt) = Ai;
K(:,1,ialt) = Ki;
Omega(:,:,ialt) = Omegai;
E(:,:,ialt) = Ei;
end

end % of primary function -----------------------------------------------------------------------------------