function output = project(w,data,nproj,varargin)
%
% PROJECT  Extend data with VAR-based optimal linear projection forward and/or backward.
%
% Syntax:
%   dpack = project(w,dpack,nproj,...)
% Arguments:
%  dpack cell; w rvar|svar; nproj numeric
% Options:
%   'backward' logical (true)
%   'deviation' logical (false)
%   'forward' logical (true)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'backward',true,...
  'deviation',false,...
  'forward',true,...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
ndata = size(data{1},3);

nloop = max([nalt,ndata]);
d = nalt - ndata;
if d > 0
  data{1} = data{1}(:,:,ndata*ones([1,d]));
  data{3} = data{3}(:,:,ndata*ones([1,d]));
end

% forward projection
if options.forward == true
  project_();
  data{3} = [data{3},zeros([ny,nproj,nloop])];
  data{4} = [data{4},data{4}(end)+(1:nproj)];
end

% backward projection
if options.backward == true
  B = w.B;
  w.B = [];
  w = backward(w);
  data{1}(:,:,:) = data{1}(:,end:-1:1,:);
  project_();
  data{1}(:,:,:) = data{1}(:,end:-1:1,:);
  data{3} = [zeros([ny,nproj,nloop]),data{3}];
  data{4} = [data{4}(1)+(-nproj:-1),data{4}];
  w.B = B;
end  

data{2} = zeros([0,length(data{4}),nloop]);

% end of function body --------------------------------------------------------------------------------------

  function project_() % nested function ---------------------------------------------------------------------
  last = size(data{1},2);
  for iloop = 1 : nloop
    if iloop <= nalt, A = w.A(:,:,iloop); K = w.K(:,1,iloop); end
    for t = last + (1 : nproj)
      data{1}(:,t,iloop) = A*vec(data{1}(:,t-1:-1:t-p,iloop));
      if options.deviation == false, data{1}(:,t,iloop) = data{1}(:,t,iloop) + w.K; end
    end
  end
  end % end of nested function ------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------