function Y = resample(w,source,ndraw,varargin)
%
% RESAMPLE  Resample from VAR data.
%
% Syntax:
%   Y = resample(w,source,ndraw,...)
% Arguments:
%   Y numeric; w VAR; source cell; ndraw numeric
% Options:
%   'randomise' logical (false) If true initial condition is randomised, otherwise fixed to pre-sample values.
%   'wild' logical (false) If true bootstrap is wild, otherwise Efron.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'randomise',false,...
  'wild',false,...
};
options = passopt(default,varargin{:});

if ~iscell(source) || ~isnumeric(ndraw), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
if nalt > 1, error_(11,{'RESAMPLE'}); end

if options.randomise == false
  sample = getsample([source{1};source{3}]);
  start = find(sample,1) + p;
  finish = find(sample,1,'last');
else
  sample = getsample(source{3});
  start = find(sample,1);
  finish = find(sample,1,'last');
end

nper = finish - start + 1;
Y = source{1}(:,:,ones([1,ndraw]));

for idraw = 1 : ndraw
  if options.wild == false
    % Efron bootstrap
    % setting draw = 0 : nper-1; would reproduce sample
    % uniform on [0,nper-1]
    draw = floor(nper*rand([1,nper]));
    E(:,start:finish) = source{3}(:,start+draw);
  else
    % wild bootstrap
    % setting draw = ones([1,nper]); would reproduce sample
    draw = randn([1,nper]);
    E(:,start:finish) = source{3}(:,start:finish).*draw(ones([1,ny]),:);
  end
  if ~isempty(w.B), E(:,start:finish) = w.B*E(:,start:finish); end
  for t = start : finish
    Y(:,t,idraw) = w.A*vec(Y(:,t-1:-1:t-p,idraw)) + E(:,t) + w.K;
  end
end
Y = permute(Y,[2,1,3]);

end % of primary function -----------------------------------------------------------------------------------