function code = sprintf(w,varargin)
%
% SPRINTF  Print SVAR as model code.
% 
% Syntax:
%   ch = sprintf(w,...)
% Arguments:
%   ch char; w svar
% Options:
%   'decimal' numeric (empty)
%   'declare' logical (false)
%   'enames' cellstr (empty)
%   'format' char ('%+.16e')
%   'ynames' cellstr (empty)
%   'tolerance' numeric (getrealsmall())
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'decimal',[],...
  'declare',false,...
  'enames',{},...
  'format','%+.16e',...
  'ynames',{},...
  'tolerance',getrealsmall(),...
};
options = passopt(default,varargin{:});

if ~isempty(options.decimal), options.format = ['%+.',sprintf('%g',options.decimal),'e']; end

% function body ---------------------------------------------------------------------------------------------

if isempty(w.B), error_(15); end

[ny,p] = dim(w);
if isempty(options.ynames)
  for i = 1 : ny
    options.ynames{end+1} = sprintf('y%g{t}',i);
  end
else
 for i = 1 : ny
    if isempty(strfind(options.ynames{i},'{t}')), options.ynames{i} = sprintf('%s{t}',options.ynames{i}); end
  end
end  
if isempty(options.enames)
  for i = 1 : ny, options.enames{end+1} = sprintf('e%g',i); end
end

options.ynames = strrep(options.ynames,'{t}','{%+g}');
for i = 1 : ny
  for j = 0 : p, ynames{i}{1+j} = sprintf(options.ynames{i},-j); end
  ynames{i}{1} = strrep(ynames{i}{1},'{+0}','');
end
enames = options.enames;

[A,B,K] = get(w,'A','B','K');
for eq = 1 : ny
  equation{eq} = [ynames{eq}{1},' = '];
  if abs(K(eq)) > options.tolerance
    equation{eq} = [equation{eq},' ',sprintf(options.format,K(eq))];
  end
  for t = 1 : p
    for y = 1 : ny
      if abs(A(eq,y,1+t)) > options.tolerance
        equation{eq} = [equation{eq},' ',sprintf(options.format,-A(eq,y,1+t)),'*',ynames{y}{1+t}];
      end
    end
  end
  for e = 1 : ny
    if abs(B(eq,e)) > options.tolerance
      equation{eq} = [equation{eq},' ',sprintf(options.format,B(eq,e)),'*',enames{e}];
    end
  end
end
code = sprintf('%s;\n',equation{:});

if options.declare == true
  options.ynames = regexprep(options.ynames,'\{.*\}','');
  decl = sprintf('@variables:transition%s\n@variables:residual%s\n@equations:transition\n',sprintf(' %s',options.ynames{:}),sprintf(' %s',options.enames{:}));
  code = [decl,code];
end

end % of primary function -----------------------------------------------------------------------------------