function [Phi,Psi] = srf(w,nper)
%
% SRF  Impulse response function (or VMA representation) of VAR.
%
% Syntax:
%   [Phi,Psi] = srf(w,nper)
% Arguments:
%   Phi numeric; Psi numeric; w svar; nper numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~isnumeric(nper), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

if isempty(w.B), error_(16); end

[ny,p,nalt] = dim(w);
[T,R,K,Z,H,D,Za,Omega] = sspace(w);
Phi = nan([ny,ny,nper,nalt]);
for ialt = 1 : nalt
  phi = srf(T(:,:,ialt),R(:,:,ialt),[],[],[],[],Za(:,:,ialt),[],nper);
  Phi(:,:,:,ialt) = phi(1:ny,:,:);
end
if nargout > 1, Psi = cumsum(Phi,3); end

end % of primary function -----------------------------------------------------------------------------------