function [T,R,K,Z,H,D,U,Omega,w] = sspace(w)
%
% SSPACE  Quasi-triangular state-space form of VAR.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
if isempty(w.TZ), w = schur_(w); end

% first-order companion form
[A,B,K,Omega] = companion(w);

% quasi-triangular state space
for ialt = 1 : nalt
  T(:,:,ialt) = w.TZ{1}(:,:,ialt);
  U(:,:,ialt) = w.TZ{2}(:,:,ialt);
  R(:,:,ialt) = transpose(U(:,:,ialt))*B(:,:,ialt);
  if ~isempty(U), K(:,:,ialt) = transpose(U(:,:,ialt))*K(:,1,ialt); end
  Z(:,:,ialt) = eye([ny,ny*p]);
end
H = zeros([ny,ny,nalt]);
D = zeros([ny,1,nalt]);

end % of primary function -----------------------------------------------------------------------------------