function [w,data] = subvar(w,data,selection)
%
% SUBVAR  Reduce VAR to a subset of variables.
% 
% Syntax:
%   w = subvar(w,selection)
% Arguments:
%   w rvar|svar; selection logical|numeric
% 
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);

if ~islogical(selection)
  aux = selection;
  selection = false([1,ny]);
  selection(aux) = true;
end

A = var2poly(w.A);
w.A = poly2var(A(selection,selection,:,:));
w.K = w.K(selection,1,:);
if ~isempty(w.B)
  w.B = w.B(selection,selection,:,:);
  w.Omega = zeros(size(w.B));
  for ialt = 1 : nalt, w.Omega(:,:,ialt) = w.B(:,:,ialt)*transpose(w.B(:,:,ialt)); end
else
  w.Omega = w.Omega(selection,selection,:);
end
w.aic = NaN;
w.sbc = NaN;
w.nhyper = NaN;
if ~isempty(w.TZ)
  w.TZ = {};
  w = eig(w);
end
if ~isempty(data) && nargout > 1
  data{1} = data{1}(selection,:,:);
  data{3} = data{3}(selection,:,:);
end

end % of primary function ------------------------------------------------------------------------------------