function [S,D] = xsf(w,freq)
%
% XSF  Power spectrum and spectral density function.
%
% Syntax:
%   [S,D] = xsf(w,freq)
% Arguments:
%   S,D cell; w rvar; freq numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~isnumeric(freq), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
freq = vech(freq);
for ialt = 1 : nalt, S{1}(:,:,:,ialt) = xsfvar(w.A(:,:,ialt),w.Omega(:,:,ialt),freq); end
S{2} = freq;

% power spectrum to spectral density conversion
if nargout > 1
  C = acf(w);
  D = {xsf2sdf(S{1},C),S{2}};
end

end % of primary function -----------------------------------------------------------------------------------