function [C,R] = acf(x,dates,varargin)
%
% ACF  Sample autocovariance and autocorrelation function.
%
% Syntax:
%   [C,R] = acf(x,dates,...)        
% Arguments:
%   C numeric; R numeric; x tseries; dates numeric|Inf
% Options:
%   'order' numeric (0)
%   'smallsample' logical (true)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if nargin < 2
  sample = getsample(permute(x.data,[2,1,3]));
  data = x.data(sample,:);
else
  data = getdata_(x,dates);
end
nalt = size(data,3);
for ialt = 1 : nalt
  C(:,:,:,ialt) = acovfsmp(data(:,:,ialt),varargin{:});
  if nargout > 1, R(:,:,:,ialt) = cov2corr(C(:,:,:,ialt)); end
end

end % of primary function -----------------------------------------------------------------------------------