function x = arf(x,A,z,range)
%
% ARF  Scalar autoregressive filter.
%
% x = arf(x,A,z,range)
% Arguments:
%   x tseries; A numeric; z tseries|numeric; range numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~istseries(x) || ~isnumeric(A) || (~isnumeric(z) && ~istseries(z)) || ~isnumeric(range)
  error('Incorrect type of input argument(s).');
end

% function body ---------------------------------------------------------------------------------------------

A = vech(A);
order = length(A) - 1;

if isinf1(range), range = x.start + (0 : size(x.data,1)-1);
  else range = range(1) : range(end); end

if istseries(z)
  zdata = getdata_(z,range);
else
  if isempty(z), z = 0; end
  zdata = z(ones([1,length(range)]));
end

if A(1) ~= 1
  zdata = zdata / A(1);
  A = A / A(1);
end

xdata = getdata_(x,range(1)-order:range(end));
[xdata,dim,nper] = reshape_(xdata);
lags = -1 : -1 : -order;
for i = 1 : size(xdata,2)
  for t = 1+order : nper
    xdata(t,i) = -A(2:end)*xdata(t+lags,i) + zdata(t-order);
  end
end

xdata = reshape_(xdata(order+1:end,:),dim);
x = setdata_(x,range,xdata);
x = cut_(x);

end % of primary function -----------------------------------------------------------------------------------