function x = cat(n,varargin)
%
% CAT  Splice time series in n-th dimension.
%
% Syntax:
%   x = cat(n,x,<y>)
% Arguments:
%   x tseries; 
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if length(varargin) == 1
  % this is because Matlab strangely calls horzcat(x) first for [x;y]
  x = varargin{1};
  return
else
  % check classes and frequencies
  inputs = catcheck_(varargin{:});
end

% remove inputs with zero size in all higher dimensions
remove = false(size(inputs));
for i = 1 : length(inputs)
  si = size(inputs{i});
  if all(si(2:end) == 0), remove(i) = true; end
end
inputs(remove) = [];

if isempty(inputs)
  x = tseries([],[]);
  return
end

ninput = length(inputs);
% find minimum start and maximum end
realsmall = getrealsmall();
start = nan([1,ninput]);
finish = nan([1,ninput]);
for i = 1 : ninput
  start(i) = inputs{i}.start;
  finish(i) = start(i) + size(inputs{i}.data,1) - 1;
end
minstart = min(start(~isnan(start)));
maxfinish = max(finish(~isnan(finish)));
% expand data with pre- or post-sample NaNs
if ~isempty(minstart)
  for i = find(start ~= minstart | finish ~= maxfinish)
    dim = size(inputs{i}.data);
    if isnan(inputs{i}.start), inputs{i}.data = nan([round(maxfinish-minstart+1),dim(2:end)]);
      else, inputs{i}.data = [nan([round(start(i)-minstart),dim(2:end)]);inputs{i}.data;nan([round(maxfinish-finish(i)),dim(2:end)])]; end
  end
end

x = inputs{1};
x.start = minstart;
for i = 2 : ninput
  x.data = cat(n,x.data,inputs{i}.data);
  x.comment = cat(n,x.comment,inputs{i}.comment);
end

end % of primary function -----------------------------------------------------------------------------------