function varargout = get(x,varargin)
%
% GET  Get time series attributes.
%
% Syntax:
%   [<value>] = get(x,<attrib>)
% Arguments:
%   value depends on attrib; x tseries; attrib char
% Attributes:
%   'comment' returns char
%   'freq' returns numeric
%   'end' returns numeric
%   'range' returns numeric
%   'start' returns numeric
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin-1 ~= nargout && nargout > 0, error('Invalid number of input or output arguments.'); end
if ~iscellstr(varargin(1:2:end)), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

invalid = cell([1,0]);
for i = 1 : nargin-1
  [varargout{i},valid] = get_(varargin{i});
  if valid == false, invalid{end+1} = varargin{i}; end
end

if ~isempty(invalid), warning(sprintf('Unrecognised attribute: ''%s''. ',invalid{:})); end

  function [out,valid] = get_(attribute) % nested function ------------------------------------------------

  attribute(isspace(attribute)) = '';
  attribute = lower(attribute);
    
  tokens = regexp(attribute,'(dat2char|dat2str)\((.*?)\)','tokens');
  if ~isempty(tokens)
    attribute = tokens{1}{2};
    transform = str2func(tokens{1}{1});
  else
    transform = @(x) x;
  end

  valid = true;
  switch attribute
  case {'range','first2last','start2end','first:last','start:end'}
    out = transform(range_(x));
  case {'start','startdate','first'}
    out = transform(x.start);
  case {'end','enddate','last'}
    out = transform(x.start + size(x.data,1) - 1);
  case {'freq','frequency','periodicity'}
    if isempty(x.start), out = NaN;
      else out = datfreq(x.start); end
  case {'data','value','values'}
    out = x.data;
  case 'comment'
    out = comment(x);
  otherwise
    valid = false;
    out = [];
  end % of switch
  
  end % of nested function ----------------------------------------------------------------------------------
  
end % of primary function -----------------------------------------------------------------------------------