function [hAx,hLeft,hRight,range] = plotyy(varargin)
%
% PLOTYY  Line plot function with LHS and RHS axes for time series.
%
% [hax,hleft,hright,plotrng] = plot(uleft,uright)
% [hax,hleft,hright,plotrng] = plot(uleft,uright,dateformat)
% [hax,hleft,hright,plotrng] = plot(rng,uleft,uright)
% [hax,hleft,hright,plotrng] = plot(rng,uleft,uright,dateformat)
%
% Arguments:
%   hax: numeric, hleft: numeric, hright: numeric, plotrng: numeric,
%   uleft: tseries, uright: tseries, rng: numeric, dateformat: char
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

switch nargin
case 2
  xLeft = varargin{1};
  xRight = varargin{2};
  range = inf;
  dateformat = '';
case 3
  if isa(varargin{end},'tseries')
    range = varargin{1};
    xLeft = varargin{2};
    xRight = varargin{3};
    dateformat = '';
  else
    range = inf;
    xLeft = varargin{1};
    xRight = varargin{2};
    dateformat = varargin{3};
  end
case 4
  range = varargin{1};
  xLeft = varargin{2};
  xRight = varargin{3};
  dateformat = varargin{4};
otherwise
  error('Incorrect number of input argument(s).');    
end

% -----function PLOTYY body----- %

handle = [];
aux = find(dateformat == '|');
if ~isempty(aux)
  dateformat = dateformat(aux+1:end);
end
if isempty(dateformat)
  irisconfig = load('irisconfig.mat');
  dateformat = irisconfig.plotdateformat;
end

nLeft = length(xLeft);
nRight = length(xRight);
if nLeft > 1
  xLeft = vech(xLeft);
elseif nLeft == 0
  error('Unable to use PLOTYY when one of time series arrays is empty.');
end
if nRight > 1
  xRight = vech(xRight);
elseif nRight == 0
  error('Unable to use PLOTYY when one of time series arrays is empty.');
end
[array,range] = double([xLeft,xRight],range);
arrayLeft = array(:,1:nLeft);
arrayRight = array(:,nLeft+1:end);
if isempty(range)
  [hAx,hLeft,hRight] = plotyy([0,1],NaN([2,nLeft]),[0,1],NaN([2,nRight]));
  set(hAx,'xlim',[0,1],'xtick',[0 1],'ylim',[0 1],'ytick',[0 1]);
  set(hAx,'xticklabel',strvcat('NaN','NaN'),'yticklabel',strvcat('NaN','NaN'));
  return
end

[year,per,freq] = dat2ypf(range);
freq = freq(1);

delete(get(gca,'children'));

time = dat2dec(range);
[hAx,hLeft,hRight] = plotyy(time,arrayLeft,time,arrayRight);
if ~isnan(freq)
  set(hAx,'xlim',time([1 end]));
  set(hAx,'xlimmode','manual');
end
setdatetick(hAx(1),freq,dateformat,Inf);
setdatetick(hAx(2),freq,dateformat,Inf);
set(hAx,'ycolor','black');

set(hAx,'tag','tseries','userdata',freq);

end