function x = freq2freq_(x,range,fromfreq,tofreq,fn)
%
% TSERIES/PRIVATE/FREQ2FREQ_  General frequency conversion for time series.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if isnan(x.start) && isempty(x.data), return, end

if datfreq(x.start) ~= fromfreq
  error('Input argument must be a %s time series.',cellref({'annual','semi-annual','quarterly','monthly'},fromfreq == [1,2,4,12]));
end
if isempty(range), x = empty(x); return, end
if ~isinf1(range), x = resize(x,range); end

fromdate = cellref({@yy,@zz,@qq,@mm},fromfreq == [1,2,4,12]);
todate = cellref({@yy,@zz,@qq,@mm},tofreq == [1,2,4,12]);

startyear = dat2ypf(get(x,'start'));
endyear = dat2ypf(get(x,'end'));

fromdata = getdata_(x,fromdate(startyear,1):fromdate(endyear,fromfreq));
[fromdata,dim,nper] = reshape_(fromdata);
factor = fromfreq/tofreq;
todata = nan([nper/factor,size(fromdata,2)]);
for i = 1 : size(fromdata,2), todata(:,i) = fn(reshape(fromdata(:,i),[factor,nper/factor])); end
todata = reshape_(todata,dim);

x.start = todate(startyear,1);
x.data = todata;
x = cut_(x);

end % of primary function ----------------------------------------------------------------------------------