function [xsa,xsfactor,xtrend,ss] = seasf(x,range,varargin)
%
% SEASF  Structural additive seasonal filter for quarterly series.
%
% [xsa,xsfactor,xtrend] = seasf(x,range,...)
% Arguments:
%   xsa tseries, xsfactor tseries, xtrend tseries, x tseries, range numeric
% Options:
%   'log' logical (false)
%   'stdseason' numeric (1)
%   'stdnoise' numeric (1)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

for i = 1 : length(x)
  if get(x(i),'freq') ~= 4, error('SEASF only works with quarterly time series.'); end
end
  
if nargin < 2, range = Inf;
  else range = setrange(range); end

% function body ---------------------------------------------------------------------------------------------

[x.data,dim] = reshape_(x.data);
[x.data,range] = getdata_(x,range);
x.start = range(1);
xsa = x;
xsfactor = x;
xtrend = x;
[xsa.data(:,:),xsfactor.data(:,:),xtrend.data(:,:),ss] = seasf(x.data,varargin{:});
xsa.data = reshape_(xsa.data,dim);
xsfactor.data = reshape_(xsfactor.data,dim);
xtrend.data = reshape_(xtrend.data,dim);

end % of primary function -----------------------------------------------------------------------------------