function x = tseries(dates,data,cmt)
%
% TSERIES  Create new time series object.
%
% Syntax:
%   x = tseries()
%   x = tseries(s)
%   x = tseries(dates,data)
%   x = tseries(dates,data,cmt)
% Arguments:
%   x teries; s struct; dates numeric; data numeric|function_handle|char; cmt char|cellstr
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin < 1, dates = []; end
if nargin < 2, data = nan([size(dates,1),1]); end
if nargin < 3, cmt = ''; end

if (~isnumeric(dates) && ~isstruct(dates)) || (~isnumeric(data) && ~islogical(data) && ~ischar(data) && ~isa(data,'function_handle')) || (~ischar(cmt) && ~iscellstr(cmt))
  error('Incorrect type of input argument(s).');
end

% function body ---------------------------------------------------------------------------------------------

x = empty_();
flag = issingle(data);

if nargin == 0, x = class(x,'tseries'); return, end

if isstruct(dates)
  for f = vech(fieldnames(x))
    try, x.(f{1}) = dates.(f{1}); catch, end
  end
  x = cut_(class(x,'tseries'));
  return
end

dates = dates(:);
if ischar(data) || isa(data,'function_handle')
  data = feval(data,[length(dates),1]);
elseif isnumeric(data)
  % squeeze if scalar time series entered as other-than-columnwise vectors
  if sum(size(data) > 1) == 1 && length(data) > 1 && length(dates) > 1
    data = data(:);
  % expand scalar data point to match more than one dates
  elseif length(data) == 1 && length(dates) > 1
    data = data(ones(size(dates)));
  end
end
[x.start,x.data,dim] = init_(dates,data);
x.comment = cell([1,dim]);
if ischar(cmt), x.comment(:) = {cmt};
  else x.comment(:) = cmt(:); end
x = cut_(class(x,'tseries'));
if flag == true, x.data = single(x.data); end

  function x = empty_() % nested function -------------------------------------------------------------------
  x.start = NaN;
  x.data = zeros([0,1]);
  x.comment = {''};
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------

% size(dates,1) must match size(data,1) even though it is zero
% the only exception is if size(dates,1) > 1 && size(data,1) == 1 then data is expanded accordingly