function x = vertcat(varargin)
%
% VERTCAT  Vertical concatenation of time series.
%
% Syntax:
%   x = vertcat(x,<y>)
%   x = [x;<y>]
% Arguments
%   x tseries; y tseries
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

if length(varargin) == 1
  x = varargin{1};
  return
else
  % check classes and frequencies
  inputs = catcheck_(varargin{:});
end

ninput = length(inputs);
x = inputs{1};
[x.data,xdim] = reshape_(x.data);
for i = 2 : ninput
  [ydata,ydim] = reshape_(inputs{i}.data);
  x2 = size(x.data,2);
  y2 = size(ydata,2);
  if x2 ~= y2
    if x2 == 1 && y2 ~= 1
      x.data = x.data(:,ones([1,y2]));
      xdim = ydim;
    elseif x2 ~= 1 && y2 == 1
      ydata = ydata(:,ones([1,x2]));
    else
      error('Incorrect size of vector time series.');
    end
  end
  x = setdata_(x,inputs{i}.start+(0:size(ydata,1)-1),ydata);
end
x.data = reshape_(x.data,xdim);
x.comment = inputs{end}.comment;
x = cut_(x);

end % of primary function -----------------------------------------------------------------------------------