function varargout = x12(x,range,varargin)
%
% X12  Census X12-ARIMA procedure for trend-cycle plus seasonal plus irregular decomposition.
%
% Syntax:
%   [c1,...,cn,outputfile,errorfile] = x12(x)
%   [c1,...,cn,outputfile,errorfile] = x12(x,rng,...)
% Arguments:
%   c1,...,cn tseries; x tseries; rng double
% Options:
%   'arima' logical (false)
%   'display' logical (false)
%   'mode' char ('sign')
%   'output' char ('d11')
%   'tdays' logical (false)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'arima',false,...
  'display',false,...
  'method',[],...
  'mode','sign',...
  'output','d11',...
  'tdays',false,...
};
options = passopt(default,varargin{:});

if nargin > 1 && ~isnumeric(range), error('Incorrect type of input argument(s).'); end

if nargin < 2, range = Inf;
  else, range = setrange(range); end

% function body ---------------------------------------------------------------------------------------------

[x.data,dim] = reshape_(x.data);
[data,range] = getdata_(x,range);

output = regexp(options.output,'[a-zA-Z]\d\d','match');
noutput = length(output);
[aux{1:noutput},varargout{noutput+(1:2)}] = x12(data,range(1),varargin{:});

for i = 1 : noutput, varargout{i} = tseries(range,reshape_(aux{i},dim)); end

end % of primary function -----------------------------------------------------------------------------------