function string = dat2str(dat,varargin)
%
% DAT2STR  Date to cellstr conversion.
%
% Syntax:
%   s = dat2str(dat,...)
% Arguments:
%   s cellstr, dat numeric
% Options:
%   'dateformat' char (irisconfig.dateformat)
%   'freqletters' char (irisconfig.freqletters)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

irisconfig = load('irisconfig.mat');

oldSyntax = false;

if nargin > 1 && ~strcmp(varargin{1},'dateformat') && ~strcmp(varargin{1},'freqletters')
  oldSyntax = true;
end

if oldSyntax == true
  options.dateformat = varargin{1};
  if nargin > 2
    options.freqletters = varargin{2};
  else
    options.freqletters = irisconfig.freqletters;
  end
else
	default = {
	  'dateformat',irisconfig.dateformat,...
	  'freqletters',irisconfig.freqletters,...
	};
	options = passopt(default,varargin{1:end});
end

% function body ---------------------------------------------------------------------------------------------

if isempty(options.dateformat)
  options.dateformat = irisconfig.dateformat;
end
if ~isa(options.freqletters,'char') || length(options.freqletters) ~= 4
  options.freqletters = irisconfig.freqletters;
end

[year,per,freq] = dat2ypf(dat);

string = cell(size(year));

n = length(vech(year));
for k = 1 : n
  string{k} = datstr(year(k),per(k),freq(k),options.dateformat,options.freqletters);
end

end % of primary function -----------------------------------------------------------------------------------