function idr = str2dat(string,varargin)
%
% STR2DAT  Cellstr to date conversion.
%
% Syntax:
%   d = str2dat(s,...)
% Arguments: 
%   d: numeric; s: cellstr|char
% Options:
%   'freqletters' char (irisconfig.freqletters)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

config = load('irisconfig.mat');

oldSyntax = false;

if nargin > 1 && ~strcmp(varargin{1},'freqletters') && ~strcmp(varargin{1},'dateformat')
  options.freqletters = varargin{1};
else
	default = {
	  'freqletters',config.freqletters,...
	};
	options = passopt(default,varargin{1:end});
end

% function body ---------------------------------------------------------------------------------------------

if ~isa(options.freqletters,'char') || length(options.freqletters) ~= 4
  options.freqletters = config.freqletters;
end

freqletters = lower(options.freqletters);
if ischar(string), string = {string}; end
string = lower(string);
format = ['%n','%[',freqletters,']','%n'];
idr = nan(size(string));
for i = 1 : length(string(:))
  idr(i) = scan2idr_(string{i},format,freqletters);
end

invalid = vech(string(isnan(idr)));
if ~isempty(invalid), error('Unable to interpret date string: %s.\n',invalid{:}); end

end % of primary function -----------------------------------------------------------------------------------

  function idr = scan2idr_(string,format,freqletters) % subfunction -----------------------------------------

  idr = NaN;
  [x,count] = textscan(string,format);

  if count == length(string)
    if isempty(x{2})
      idr = x{1}; % indeterminate frequency
    else
      if isempty(x{3}), x{3} = 1; end
      freq = [1,2,4,12];
      index = find(freqletters == x{2}{1});
      if length(index) == 1, idr = datcode(x{1},x{3},freq(index)); end
    end
  end

  end % of subfunction --------------------------------------------------------------------------------------