function [dbase,list0,list,flag] = dbbatch(dbase0,namemask,exprmask,varargin)
%
% DBBATCH  Batch processing and/or creation of database fields.
%
% Syntax:
%   [dbase,list0,list,flag] = dbbatch(dbase0,name,expr,...)
% Arguments:
%   dbase struct; list0 cell; list cell; flag logical; dbase0 struct; name char; expr char
% Options:
%  'classfilter' char|Inf (Inf)
%  'merge' logical (true)
%  'namefilter' char|Inf (Inf)
%  'namelist' cellstr (empty)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~isstruct(dbase0) || ~ischar(namemask) || ~ischar(exprmask) || ~iscellstr(varargin(1:2:nargin-3))
  error('Incorrect type of input argument(s).');
end  

default = {
  'append',true,...
  'classfilter',Inf,...
  'namefilter',Inf,...
  'namelist',{},...
  'merge',true,...
};
options = passopt(default,varargin{:});

if any(strcmpi(varargin(1:2:end),'append'))
  warning('iris:obsolete','APPEND is an obsolete option. Use MERGE instead.');
  options.merge = options.append;
end

% function body ---------------------------------------------------------------------------------------------

[list0,list,expr] = dbquery(dbase0,namemask,exprmask,options.classfilter,options.namefilter,options.namelist);
expr = strrep(expr,'"','''');

flag = true;
if options.merge == true, dbase = dbase0;
  else, dbase = struct(); end

errorlist = {};
warnlist = {};
for i = 1 : length(list0)
  try
    warning('');
    value = evalin('caller',expr{i});
    msg = lastwarn();
    dbase.(list{i}) = value;
    if ~isempty(msg), warnlist(end+1) = expr(i); end
  catch
    errorlist(end+1:end+2) = {expr{i},sprintf('Matlab says: %s',geterrormsg())};
  end
end
if ~isempty(errorlist), flag = false; warning_(4,errorlist); end
if ~isempty(warnlist), flag = false; warning_(5,warnlist); end
  
end % of primary function -----------------------------------------------------------------------------------