function [x,flag,invalid] = dbfun(fn,d,varargin)
%
% DBFUN  Apply function to each database field.
%
% Syntax:
%   [d,flag,invalid] = dbfun(fn,d,...)
% Arguments:
%   d struct; flag logical; invalid cellstr; fn function_handle|char
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~(ischar(fn) || isa(fn,'function_handle')) || ~isstruct(d)
  error('Incorrect type of input argument(s).');
end

% function body ---------------------------------------------------------------------------------------------

list = fieldnames(d);

flag = true;
x = struct();
invalid = {};
for i = 1 : length(list)
  try
    arglist = {d.(list{i})};
    for j = 1 : length(varargin)
      arglist(end+1) = {varargin{j}.(list{i})};
    end
    x.(list{i}) = fn(arglist{:});
  catch
    x.(list{i}) = NaN;
    invalid{end+1} = list{i};
  end
end

if ~isempty(invalid)
  flag = false;
  warning_(3,invalid);
end

end % of primary function -----------------------------------------------------------------------------------