function S = xsfvar(A,Omega,freq,filter,applyto,order)
%
% FREQ-DOMAIN/XSFVAR  Cross-spectrum function for VAR processes.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

isfilter = nargin > 3;

[ny,p] = size(A);
p = p/ny;
A = reshape(A,[ny,ny,p]);
nfreq = length(freq);

if isfilter == true, n = 1+order;
  else n = nfreq; end
S = zeros([ny,ny,n]);
e = exp(complex(0,-1)*(1:p));
for i = 1 : nfreq
  lambda = freq(i);
  F = eye(ny);
  for j = 1 : p, F = F - exp(-1i*j*lambda)*A(:,:,j); end
  s = F \ Omega / ctranspose(F);
  if isfilter == true
    s(applyto,:) = filter(i)*s(applyto,:);            
    s(:,applyto) = s(:,applyto)*conj(filter(i));            
    S(:,:,1) = S(:,:,1) + s;
    for j = 1 : order, S(:,:,1+j) = S(:,:,1+j) + s*exp(1i*freq(i)*j); end
  else
    S(:,:,i) = s;
  end
end

if isfilter == false
  % if isfilter == true we should divide S by 2*pi here but then multiply it by 2*pi afterwards in ACF: we therefore skip it in both steps
  S = S/(2*pi);
  for i = 1 : size(S,1), S(i,i,:) = real(S(i,i,:)); end
end


end % of primary function -----------------------------------------------------------------------------------