function [Theta,Upsilon] = fevd(T,R,K,Z,H,D,Za,Omega,nper)
%
% TIME-DOMAIN/FEVD  Forecast error variance decomposition for general state space.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

% function body ---------------------------------------------------------------------------------------------

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);
n = ny + nf + nb;

[M,Phi] = fmse(T,R,K,Z,H,D,Za,Omega,nper);
Theta = cumsum(Phi.^2,3); % absolute MSE
Upsilon = zeros(size(Theta)); % relative contribution of innovations
varvec = vech(diag(Omega));
varvec = varvec(ones([1,n]),:);
warning('off','MATLAB:divideByZero');
for i = 1 : nper  
  Theta(:,:,i) = Theta(:,:,i) .* varvec;
  Mdiag = diag(M(:,:,i));
  Mdiag = Mdiag(:,ones([1,ne]));
  Upsilon(:,:,i) = Theta(:,:,i) ./ Mdiag;
end
warning('on','MATLAB:divideByZero');

end % of primary function -----------------------------------------------------------------------------------