function h = grayarea(varargin)
%
% GRAYAREA  Fill gray area between start-date and end-date.
%
% Syntax:
%   h = grayarea(hax,range,...)
% Arguments:
%   h numeric; hax numeric; range numeric
% Options:
%   'grade' numeric (0.8)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin > 1 && isnumeric(varargin{2})
  hax = varargin{1}; 
  range = varargin{2};
  varargin(1:2) = [];
else
  hax = gca; 
  range = varargin{1};
  varargin(1) = [];
end

default = {
  'grade',0.8,...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

if isempty(range), return
  else xstart = range(1); xend = range(end); end

status = get(hax,'nextplot');
set(hax,'nextplot','add');
ylim = get(hax,'ylim');
set(hax,'ylimmode','manual');
set(hax,'layer','top');
if strcmp(get(hax,'tag'),'tseries'), xlim = dat2dec([xstart*[1,1],xend*[1,1]]);
  else xlim = [xstart*[1,1],xend*[1,1]]; end
h = fill(xlim,[ylim(1),ylim(2),ylim(2),ylim(1)],options.grade*[1,1,1]);
set(h,'linestyle','none');
ch = get(hax,'children');
set(hax,'children',[ch(2:end);ch(1)]);
set(hax,'nextplot',status);

end % of primary function -----------------------------------------------------------------------------------